@@ -1,4 +1,4 @@
-/* $Id: l16.c 4987 2015-03-03 02:41:27Z ming $ */
+/* $Id: l16.c 5632 2017-07-27 06:45:48Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -312,6 +312,7 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
 	++count;
     }
 
+#if PJMEDIA_CODEC_L16_HAS_8KHZ_MONO
     if (count < *max_count) {
 	/* 8KHz mono */
 	codecs[count].type = PJMEDIA_TYPE_AUDIO;
@@ -321,7 +322,9 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
 	codecs[count].channel_cnt = 1;
 	++count;
     }
+#endif
 
+#if PJMEDIA_CODEC_L16_HAS_8KHZ_STEREO
     if (count < *max_count) {
 	/* 8KHz stereo */
 	codecs[count].type = PJMEDIA_TYPE_AUDIO;
@@ -331,6 +334,7 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
 	codecs[count].channel_cnt = 2;
 	++count;
     }
+#endif
 
 // disable some L16 modes
 #if 0
@@ -355,6 +359,7 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
     }
 #endif
 
+#if PJMEDIA_CODEC_L16_HAS_16KHZ_MONO
     if (count < *max_count) {
 	/* 16000 Hz mono */
 	codecs[count].type = PJMEDIA_TYPE_AUDIO;
@@ -364,8 +369,9 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
 	codecs[count].channel_cnt = 1;
 	++count;
     }
+#endif
 
-
+#if PJMEDIA_CODEC_L16_HAS_16KHZ_STEREO
     if (count < *max_count) {
 	/* 16000 Hz stereo */
 	codecs[count].type = PJMEDIA_TYPE_AUDIO;
@@ -375,6 +381,7 @@ static pj_status_t l16_enum_codecs( pjmedia_codec_factory *factory,
 	codecs[count].channel_cnt = 2;
 	++count;
     }
+#endif
 
 // disable some L16 modes
 #if 0
@@ -591,7 +598,7 @@ static pj_status_t  l16_parse( pjmedia_codec *codec,
 	frames[count].type = PJMEDIA_FRAME_TYPE_AUDIO;
 	frames[count].buf = pkt;
 	frames[count].size = data->frame_size;
-	frames[count].timestamp.u64 = ts->u64 + (count * data->frame_size);
+	frames[count].timestamp.u64 = ts->u64 + (count * data->frame_size >> 1);
 
 	pkt = ((char*)pkt) + data->frame_size;
 	pkt_size -= data->frame_size;
