@@ -1,4 +1,4 @@
-/* $Id: sdp_neg.c 5045 2015-04-06 06:13:51Z nanang $ */
+/* $Id: sdp_neg.c 5619 2017-07-05 03:57:53Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -38,6 +38,7 @@ struct pjmedia_sdp_neg
     pj_bool_t		  answer_was_remote;
 
     pjmedia_sdp_session	*initial_sdp,	    /**< Initial local SDP	     */
+			*initial_sdp_tmp,   /**< Temporary initial local SDP */
 			*active_local_sdp,  /**< Currently active local SDP. */
 			*active_remote_sdp, /**< Currently active remote's.  */
 			*neg_local_sdp,	    /**< Temporary local SDP.	     */
@@ -333,7 +334,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_modify_local_offer2(
      */
     pj_strdup(pool, &new_offer->origin.user, &old_offer->origin.user);
     new_offer->origin.id = old_offer->origin.id;
-    new_offer->origin.version = old_offer->origin.version + 1;
+
     pj_strdup(pool, &new_offer->origin.net_type, &old_offer->origin.net_type);
     pj_strdup(pool, &new_offer->origin.addr_type,&old_offer->origin.addr_type);
     pj_strdup(pool, &new_offer->origin.addr, &old_offer->origin.addr);
@@ -398,6 +399,18 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_modify_local_offer2(
     }
 
     /* New_offer fixed */
+#if PJMEDIA_SDP_NEG_COMPARE_BEFORE_INC_VERSION
+    new_offer->origin.version = old_offer->origin.version;
+
+    if (pjmedia_sdp_session_cmp(new_offer, neg->initial_sdp, 0) != PJ_SUCCESS)
+    {
+	++new_offer->origin.version;
+    }    
+#else
+    new_offer->origin.version = old_offer->origin.version + 1;
+#endif
+    
+    neg->initial_sdp_tmp = neg->initial_sdp;
     neg->initial_sdp = new_offer;
     neg->neg_local_sdp = pjmedia_sdp_session_clone(pool, new_offer);
 
@@ -423,11 +436,16 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_send_local_offer( pj_pool_t *pool,
 	/* If in STATE_DONE, set the active SDP as the offer. */
 	PJ_ASSERT_RETURN(neg->active_local_sdp, PJMEDIA_SDPNEG_ENOACTIVE);
 
+	/* Retain initial SDP */
+	if (neg->initial_sdp) {
+	    neg->initial_sdp_tmp = neg->initial_sdp;
+    	    neg->initial_sdp = pjmedia_sdp_session_clone(pool,
+							 neg->initial_sdp);
+	}
+
 	neg->state = PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER;
 	neg->neg_local_sdp = pjmedia_sdp_session_clone(pool, 
 						       neg->active_local_sdp);
-	neg->initial_sdp = pjmedia_sdp_session_clone(pool,
-						     neg->active_local_sdp);
 	*offer = neg->active_local_sdp;
 
     } else {
@@ -437,7 +455,6 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_send_local_offer( pj_pool_t *pool,
 	*offer = neg->neg_local_sdp;
     }
 
-    
     return PJ_SUCCESS;
 }
 
@@ -505,6 +522,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_local_answer( pj_pool_t *pool,
 	neg->neg_local_sdp = pjmedia_sdp_session_clone(pool, local);
 	if (neg->initial_sdp) {
 	    /* Retain initial_sdp value. */
+	    neg->initial_sdp_tmp = neg->initial_sdp;
 	    neg->initial_sdp = pjmedia_sdp_session_clone(pool,
 							 neg->initial_sdp);
         
@@ -520,6 +538,7 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_set_local_answer( pj_pool_t *pool,
 	}
     } else {
 	PJ_ASSERT_RETURN(neg->initial_sdp, PJMEDIA_SDPNEG_ENOINITIAL);
+	neg->initial_sdp_tmp = neg->initial_sdp;
 	neg->initial_sdp = pjmedia_sdp_session_clone(pool, neg->initial_sdp);
 	neg->neg_local_sdp = pjmedia_sdp_session_clone(pool, neg->initial_sdp);
     }
@@ -1168,18 +1187,18 @@ static pj_status_t match_offer(pj_pool_t *pool,
 			{
 			    /* Match! */
 			    if (is_codec) {
-				pjmedia_sdp_media *o, *a;
+				pjmedia_sdp_media *o_med, *a_med;
 				unsigned o_fmt_idx, a_fmt_idx;
 
-				o = (pjmedia_sdp_media*)offer;
-				a = (pjmedia_sdp_media*)preanswer;
+				o_med = (pjmedia_sdp_media*)offer;
+				a_med = (pjmedia_sdp_media*)preanswer;
 				o_fmt_idx = prefer_remote_codec_order? i:j;
 				a_fmt_idx = prefer_remote_codec_order? j:i;
 
 				/* Call custom format matching callbacks */
 				if (custom_fmt_match(pool, &or_.enc_name,
-						     o, o_fmt_idx,
-						     a, a_fmt_idx,
+						     o_med, o_fmt_idx,
+						     a_med, a_fmt_idx,
 						     ALLOW_MODIFY_ANSWER) !=
 				    PJ_SUCCESS)
 				{
@@ -1399,10 +1418,6 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_cancel_offer(pjmedia_sdp_neg *neg)
 		     neg->state == PJMEDIA_SDP_NEG_STATE_REMOTE_OFFER,
 		     PJMEDIA_SDPNEG_EINSTATE);
 
-    /* Clear temporary SDP */
-    neg->neg_local_sdp = neg->neg_remote_sdp = NULL;
-    neg->has_remote_answer = PJ_FALSE;
-
     if (neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER &&
 	neg->active_local_sdp) 
     {
@@ -1413,6 +1428,15 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_cancel_offer(pjmedia_sdp_neg *neg)
 	neg->active_local_sdp->origin.version++;
     }
 
+    /* Revert back initial SDP */
+    if (neg->state == PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER)
+	neg->initial_sdp = neg->initial_sdp_tmp;
+
+    /* Clear temporary SDP */
+    neg->initial_sdp_tmp = NULL;
+    neg->neg_local_sdp = neg->neg_remote_sdp = NULL;
+    neg->has_remote_answer = PJ_FALSE;
+
     /* Reset state to done */
     neg->state = PJMEDIA_SDP_NEG_STATE_DONE;
 
@@ -1461,12 +1485,21 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_negotiate( pj_pool_t *pool,
 	    else
 		active_ver = neg->initial_sdp->origin.version;
 
+#if PJMEDIA_SDP_NEG_COMPARE_BEFORE_INC_VERSION
+	    answer->origin.version = active_ver;
+
+	    if ((neg->active_local_sdp == NULL) || 
+		(pjmedia_sdp_session_cmp(answer, neg->active_local_sdp, 0) 
+								!= PJ_SUCCESS))
+	    {
+		++answer->origin.version;
+	    }
+#else
+	    answer->origin.version = active_ver + 1;
+#endif	    
 	    /* Only update active SDPs when negotiation is successfull */
 	    neg->active_local_sdp = answer;
 	    neg->active_remote_sdp = neg->neg_remote_sdp;
-
-	    /* Increment SDP version */
-	    neg->active_local_sdp->origin.version = ++active_ver;
 	}
     }
 
@@ -1476,7 +1509,12 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_negotiate( pj_pool_t *pool,
     /* Save state */
     neg->answer_was_remote = neg->has_remote_answer;
 
+    /* Revert back initial SDP if nego fails */
+    if (status != PJ_SUCCESS)
+	neg->initial_sdp = neg->initial_sdp_tmp;
+
     /* Clear temporary SDP */
+    neg->initial_sdp_tmp = NULL;
     neg->neg_local_sdp = neg->neg_remote_sdp = NULL;
     neg->has_remote_answer = PJ_FALSE;
 
@@ -1599,7 +1637,12 @@ PJ_DEF(pj_status_t) pjmedia_sdp_neg_fmt_match(pj_pool_t *pool,
     pjmedia_sdp_attr_get_rtpmap(attr, &a_rtpmap);
 
     if (pj_stricmp(&o_rtpmap.enc_name, &a_rtpmap.enc_name) != 0 ||
-	o_rtpmap.clock_rate != a_rtpmap.clock_rate)
+	(o_rtpmap.clock_rate != a_rtpmap.clock_rate) ||
+	(!(pj_stricmp(&o_rtpmap.param, &a_rtpmap.param) == 0 ||
+	   (a_rtpmap.param.slen == 0 && o_rtpmap.param.slen == 1 &&
+	    *o_rtpmap.param.ptr == '1') ||
+	   (o_rtpmap.param.slen == 0 && a_rtpmap.param.slen == 1 &&
+	    *a_rtpmap.param.ptr=='1'))))
     {
 	return PJMEDIA_SDP_EFORMATNOTEQUAL;
     }
