@@ -1,4 +1,4 @@
-/* $Id: mips_test.c 5170 2015-08-25 08:45:46Z nanang $ */
+/* $Id: mips_test.c 5607 2017-06-15 03:03:21Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -715,7 +715,7 @@ static pj_status_t codec_put_frame(struct pjmedia_port *this_port,
 	}
     }
 
-    return PJ_SUCCESS;
+    return status;
 }
 
 static pj_status_t codec_on_destroy(struct pjmedia_port *this_port)
@@ -1029,7 +1029,7 @@ static pj_status_t wsola_plc_get_frame(struct pjmedia_port *this_port,
 	wp->prev_lost = PJ_TRUE;
     }
 
-    return PJ_SUCCESS;
+    return status;
 }
 
 static pj_status_t wsola_plc_on_destroy(struct pjmedia_port *this_port)
@@ -1172,7 +1172,7 @@ static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port,
 					   pjmedia_frame *frame)
 {
     struct wsola_discard_port *wp = (struct wsola_discard_port*)this_port;
-    pj_status_t status;
+    pj_status_t status = PJ_SUCCESS;
 
     while (pjmedia_circ_buf_get_len(wp->circbuf) <
 		PJMEDIA_PIA_SPF(&wp->base.info) * (CIRC_BUF_FRAME_CNT-1))
@@ -1202,7 +1202,7 @@ static pj_status_t wsola_discard_get_frame(struct pjmedia_port *this_port,
 	pj_assert(status==PJ_SUCCESS);
     }
 
-    return PJ_SUCCESS;
+    return status;
 }
 
 static pj_status_t wsola_discard_on_destroy(struct pjmedia_port *this_port)
@@ -2093,7 +2093,7 @@ static pj_status_t delaybuf_get_frame(struct pjmedia_port *this_port,
 	}
     }
 
-    return PJ_SUCCESS;
+    return status;
 }
 
 static pj_status_t delaybuf_put_frame(struct pjmedia_port *this_port, 
@@ -2121,7 +2121,7 @@ static pj_status_t delaybuf_put_frame(struct pjmedia_port *this_port,
 	}
     }
 
-    return PJ_SUCCESS;
+    return status;
 }
 
 static pj_status_t delaybuf_on_destroy(struct pjmedia_port *this_port)
@@ -2365,6 +2365,7 @@ static pj_timestamp run_entry(unsigned clock_rate, struct test_entry *e)
 	    pj_assert(status == PJ_SUCCESS);
 	}
     }
+    PJ_UNUSED_ARG(status);
     pj_get_timestamp(&t1);
 
     pj_sub_timestamp(&t1, &t0);
