@@ -1,4 +1,4 @@
-/* $Id: sdp_neg_test.c 3553 2011-05-05 06:14:19Z nanang $ */
+/* $Id: sdp_neg_test.c 5619 2017-07-05 03:57:53Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -661,7 +661,11 @@ static struct test
 	    NULL,
 	    /* This is how Bob's answer should look like: */
 	    "v=0\r\n"
+#if PJMEDIA_SDP_NEG_COMPARE_BEFORE_INC_VERSION
+	    "o=bob 2808844564 2808844564 IN IP4 host.biloxi.example.com\r\n"
+#else
 	    "o=bob 2808844564 2808844565 IN IP4 host.biloxi.example.com\r\n"
+#endif
 	    "s=bob\r\n"
 	    "c=IN IP4 host.biloxi.example.com\r\n"
 	    "t=0 0\r\n"
