@@ -1,4 +1,4 @@
-/* $Id: ice_strans.c 5366 2016-06-29 05:53:10Z nanang $ */
+/* $Id: ice_strans.c 5655 2017-09-20 05:04:25Z riza $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -164,9 +164,15 @@ typedef struct pj_ice_strans_comp
 	unsigned	 err_cnt;	/**< TURN disconnected count.	*/
     } turn[PJ_ICE_MAX_TURN];
 
+    pj_bool_t		 creating;	/**< Is creating the candidates?*/
     unsigned		 cand_cnt;	/**< # of candidates/aliaes.	*/
     pj_ice_sess_cand	 cand_list[PJ_ICE_ST_MAX_CAND];	/**< Cand array	*/
 
+    pj_bool_t		 ipv4_mapped;   /**< Is IPv6 addr mapped to IPv4?*/
+    pj_sockaddr		 dst_addr;	/**< Destination address	*/
+    pj_sockaddr		 synth_addr;	/**< Synthesized dest address	*/
+    unsigned 		 synth_addr_len;/**< Synthesized dest addr len  */
+
     unsigned		 default_cand;	/**< Default candidate.		*/
 
 } pj_ice_strans_comp;
@@ -230,6 +236,7 @@ PJ_DEF(void) pj_ice_strans_cfg_default(pj_ice_strans_cfg *cfg)
 {
     pj_bzero(cfg, sizeof(*cfg));
 
+    cfg->af = pj_AF_INET();
     pj_stun_config_init(&cfg->stun_cfg, NULL, 0, NULL, NULL);
     pj_ice_strans_stun_cfg_default(&cfg->stun);
     pj_ice_strans_turn_cfg_default(&cfg->turn);
@@ -344,7 +351,12 @@ static pj_status_t add_update_turn(pj_ice_strans *ice_st,
 	    for (i=0; i<comp->cand_cnt; ++i) {
 		if (comp->cand_list[i].type == PJ_ICE_CAND_TYPE_SRFLX) {
 		    comp->default_cand = i;
-		    break;
+		    if (ice_st->cfg.af == pj_AF_UNSPEC() ||
+		        comp->cand_list[i].base_addr.addr.sa_family ==
+		        ice_st->cfg.af)
+		    {
+		        break;
+		    }
 		}
 	    }
 	}
@@ -375,7 +387,7 @@ static pj_status_t add_update_turn(pj_ice_strans *ice_st,
 	cand->type = PJ_ICE_CAND_TYPE_RELAYED;
 	cand->status = PJ_EPENDING;
 	cand->local_pref = RELAY_PREF;
-	cand->transport_id = CREATE_TP_ID(TP_TURN, idx);
+	cand->transport_id = tp_id;
 	cand->comp_id = (pj_uint8_t) comp->comp_id;
     }
 
@@ -412,8 +424,9 @@ static pj_status_t add_update_turn(pj_ice_strans *ice_st,
     comp->cand_cnt++;
 
     PJ_LOG(4,(ice_st->obj_name,
-		  "Comp %d: TURN relay candidate waiting for allocation",
-		  comp->comp_id));
+		  "Comp %d/%d: TURN relay candidate (tpid=%d) "
+		  "waiting for allocation",
+		  comp->comp_id, comp->cand_cnt-1, cand->transport_id));
 
     return PJ_SUCCESS;
 }
@@ -514,8 +527,9 @@ static pj_status_t add_stun_and_host(pj_ice_strans *ice_st,
 	///sess_add_ref(ice_st);
 
 	PJ_LOG(4,(ice_st->obj_name,
-		  "Comp %d: srflx candidate starts Binding discovery",
-		  comp->comp_id));
+		  "Comp %d: srflx candidate (tpid=%d) starts "
+		  "Binding discovery",
+		  comp->comp_id, cand->transport_id));
 
 	pj_log_push_indent();
 
@@ -544,7 +558,13 @@ static pj_status_t add_stun_and_host(pj_ice_strans *ice_st,
 	comp->cand_cnt++;
 
 	/* Set default candidate to srflx */
-	comp->default_cand = (unsigned)(cand - comp->cand_list);
+	if (comp->cand_list[comp->default_cand].type != PJ_ICE_CAND_TYPE_SRFLX
+	    || (ice_st->cfg.af != pj_AF_UNSPEC() &&
+	        comp->cand_list[comp->default_cand].base_addr.addr.sa_family
+	        != ice_st->cfg.af))
+	{
+	    comp->default_cand = (unsigned)(cand - comp->cand_list);
+	}
 
 	pj_log_pop_indent();
     }
@@ -621,9 +641,9 @@ static pj_status_t add_stun_and_host(pj_ice_strans *ice_st,
 
 	    if (cand_duplicate) {
 		PJ_LOG(4, (ice_st->obj_name,
-		       "Comp %d: host candidate %s is a duplicate",
+		       "Comp %d: host candidate %s (tpid=%d) is a duplicate",
 		       comp->comp_id, pj_sockaddr_print(&cand->addr, addrinfo,
-		       sizeof(addrinfo), 3)));
+		       sizeof(addrinfo), 3), cand->transport_id));
 
 		pj_bzero(&cand->addr, sizeof(cand->addr));
 		pj_bzero(&cand->base_addr, sizeof(cand->base_addr));
@@ -635,10 +655,23 @@ static pj_status_t add_stun_and_host(pj_ice_strans *ice_st,
 	    pj_ice_calc_foundation(ice_st->pool, &cand->foundation,
 				   cand->type, &cand->base_addr);
 
+	    /* Set default candidate with the preferred default
+	     * address family
+	     */
+	    if (comp->ice_st->cfg.af != pj_AF_UNSPEC() &&
+	        addr->addr.sa_family == comp->ice_st->cfg.af &&
+	        comp->cand_list[comp->default_cand].base_addr.addr.sa_family !=
+	        ice_st->cfg.af)
+	    {
+	        comp->default_cand = (unsigned)(cand - comp->cand_list);
+	    }
+
 	    PJ_LOG(4,(ice_st->obj_name,
-		      "Comp %d: host candidate %s added",
-		      comp->comp_id, pj_sockaddr_print(&cand->addr, addrinfo,
-						      sizeof(addrinfo), 3)));
+		      "Comp %d/%d: host candidate %s (tpid=%d) added",
+		      comp->comp_id, comp->cand_cnt-1, 
+		      pj_sockaddr_print(&cand->addr, addrinfo,
+					sizeof(addrinfo), 3),
+					cand->transport_id));
 	}
     }
 
@@ -665,6 +698,7 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
     comp = PJ_POOL_ZALLOC_T(ice_st->pool, pj_ice_strans_comp);
     comp->ice_st = ice_st;
     comp->comp_id = comp_id;
+    comp->creating = PJ_TRUE;
 
     ice_st->comp[comp_id-1] = comp;
 
@@ -693,6 +727,9 @@ static pj_status_t create_comp(pj_ice_strans *ice_st, unsigned comp_id)
 	}
     }
 
+    /* Done creating all the candidates */
+    comp->creating = PJ_FALSE;
+
     /* It's possible that we end up without any candidates */
     if (comp->cand_cnt == 0) {
 	PJ_LOG(4,(ice_st->obj_name,
@@ -831,6 +868,10 @@ static void destroy_ice_st(pj_ice_strans *ice_st)
 	      ice_st));
     pj_log_push_indent();
 
+    /* Reset callback and user data */
+    pj_bzero(&ice_st->cb, sizeof(ice_st->cb));
+    ice_st->user_data = NULL;
+
     pj_grp_lock_acquire(ice_st->grp_lock);
 
     if (ice_st->destroy_req) {
@@ -923,8 +964,8 @@ static void sess_init_update(pj_ice_strans *ice_st)
 {
     unsigned i;
 
-    /* Ignore if init callback has been called */
-    if (ice_st->cb_called)
+    /* Ignore if ICE is destroying or init callback has been called */
+    if (ice_st->destroy_req || ice_st->cb_called)
 	return;
 
     /* Notify application when all candidates have been gathered */
@@ -932,11 +973,25 @@ static void sess_init_update(pj_ice_strans *ice_st)
 	unsigned j;
 	pj_ice_strans_comp *comp = ice_st->comp[i];
 
+	/* This function can be called when all components or candidates
+	 * have not been created.
+	 */
+	if (!comp || comp->creating) {
+	    PJ_LOG(5, (ice_st->obj_name, "ICE init update: creating comp %d",
+		       (comp?comp->comp_id:(i+1)) ));
+	    return;
+	}
+
 	for (j=0; j<comp->cand_cnt; ++j) {
 	    pj_ice_sess_cand *cand = &comp->cand_list[j];
 
-	    if (cand->status == PJ_EPENDING)
+	    if (cand->status == PJ_EPENDING) {
+		PJ_LOG(5, (ice_st->obj_name, "ICE init update: "
+			   "comp %d/%d[%s] is pending",
+			   comp->comp_id, j,
+			   pj_ice_get_cand_type_name(cand->type)));
 		return;
+	    }
 	}
     }
 
@@ -1085,6 +1140,15 @@ PJ_DEF(pj_status_t) pj_ice_strans_init_ice(pj_ice_strans *ice_st,
 	    /* Must have address */
 	    pj_assert(pj_sockaddr_has_addr(&cand->addr));
 
+	    /* Skip if we are mapped to IPv4 address and this candidate
+	     * is not IPv4.
+	     */
+	    if (comp->ipv4_mapped &&
+	        cand->addr.addr.sa_family != pj_AF_INET())
+	    {
+	    	continue;
+	    }
+
 	    /* Add the candidate */
 	    status = pj_ice_sess_add_cand(ice_st->ice, comp->comp_id,
 					  cand->transport_id, cand->type,
@@ -1307,7 +1371,7 @@ PJ_DEF(pj_status_t) pj_ice_strans_start_ice( pj_ice_strans *ice_st,
 		}
 	    }
 
-	    if (count) {
+	    if (count && !comp->turn[n].err_cnt && comp->turn[n].sock) {
 		status = pj_turn_sock_set_perm(comp->turn[n].sock, count,
 					       addrs, 0);
 		if (status != PJ_SUCCESS) {
@@ -1451,9 +1515,33 @@ PJ_DEF(pj_status_t) pj_ice_strans_sendto( pj_ice_strans *ice_st,
 	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ?
 		    PJ_SUCCESS : status;
 	} else {
+    	    const pj_sockaddr_t *dest_addr;
+    	    unsigned dest_addr_len;
+
+    	    if (comp->ipv4_mapped) {
+    	    	if (comp->synth_addr_len == 0 ||
+    	    	    pj_sockaddr_cmp(&comp->dst_addr, dst_addr) != 0)
+    	    	{
+    	    	    status = pj_sockaddr_synthesize(pj_AF_INET6(),
+    	    					    &comp->synth_addr,
+    	    					    dst_addr);
+    	    	    if (status != PJ_SUCCESS)
+    	            	return status;
+
+    	    	    pj_sockaddr_cp(&comp->dst_addr, dst_addr);
+    	    	    comp->synth_addr_len = pj_sockaddr_get_len(
+    	    	    			       &comp->synth_addr);
+    	    	}
+	    	dest_addr = &comp->synth_addr;
+    	    	dest_addr_len = comp->synth_addr_len;
+    	    } else {
+    		dest_addr = dst_addr;
+    		dest_addr_len = dst_addr_len;
+    	    }
+
 	    status = pj_stun_sock_sendto(comp->stun[tp_idx].sock, NULL, data,
-					 (unsigned)data_len, 0, dst_addr,
-					 dst_addr_len);
+					 (unsigned)data_len, 0, dest_addr,
+					 dest_addr_len);
 	    return (status==PJ_SUCCESS||status==PJ_EPENDING) ?
 		    PJ_SUCCESS : status;
 	}
@@ -1471,6 +1559,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
     pj_ice_strans *ice_st = (pj_ice_strans*)ice->user_data;
     pj_time_val t;
     unsigned msec;
+    pj_ice_strans_cb cb = ice_st->cb;
 
     pj_grp_lock_add_ref(ice_st->grp_lock);
 
@@ -1478,7 +1567,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
     PJ_TIME_VAL_SUB(t, ice_st->start_time);
     msec = PJ_TIME_VAL_MSEC(t);
 
-    if (ice_st->cb.on_ice_complete) {
+    if (cb.on_ice_complete) {
 	if (status != PJ_SUCCESS) {
 	    char errmsg[PJ_ERR_MSG_SIZE];
 	    pj_strerror(status, errmsg, sizeof(errmsg));
@@ -1551,8 +1640,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
 					       PJ_ICE_STRANS_STATE_FAILED;
 
 	pj_log_push_indent();
-	(*ice_st->cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_NEGOTIATION,
-				      status);
+	(*cb.on_ice_complete)(ice_st, PJ_ICE_STRANS_OP_NEGOTIATION, status);
 	pj_log_pop_indent();
 
     }
@@ -1600,9 +1688,31 @@ static pj_status_t ice_tx_pkt(pj_ice_sess *ice,
 	    status = PJ_EINVALIDOP;
 	}
     } else if (tp_typ == TP_STUN) {
+    	const pj_sockaddr_t *dest_addr;
+    	unsigned dest_addr_len;
+
+    	if (comp->ipv4_mapped) {
+    	    if (comp->synth_addr_len == 0 ||
+    	    	pj_sockaddr_cmp(&comp->dst_addr, dst_addr) != 0)
+    	    {
+    	    	status = pj_sockaddr_synthesize(pj_AF_INET6(),
+    	    					&comp->synth_addr, dst_addr);
+    	    	if (status != PJ_SUCCESS)
+    	    	    return status;
+    	    
+    	    	pj_sockaddr_cp(&comp->dst_addr, dst_addr);
+    	    	comp->synth_addr_len = pj_sockaddr_get_len(&comp->synth_addr);
+    	    }
+	    dest_addr = &comp->synth_addr;
+    	    dest_addr_len = comp->synth_addr_len;
+    	} else {
+    	    dest_addr = dst_addr;
+    	    dest_addr_len = dst_addr_len;
+    	}
+
 	status = pj_stun_sock_sendto(comp->stun[tp_idx].sock, NULL,
 				     pkt, (unsigned)size, 0,
-				     dst_addr, dst_addr_len);
+				     dest_addr, dest_addr_len);
     } else {
 	pj_assert(!"Invalid transport ID");
 	status = PJ_EINVALIDOP;
@@ -1770,6 +1880,41 @@ static pj_bool_t stun_on_status(pj_stun_sock *stun_sock,
 				    "srflx address changed";
 		pj_bool_t dup = PJ_FALSE;
 
+		if (info.mapped_addr.addr.sa_family == pj_AF_INET() &&
+		    cand->base_addr.addr.sa_family == pj_AF_INET6())
+		{
+		    /* We get an IPv4 mapped address for our IPv6
+		     * host address.
+		     */		     
+		    comp->ipv4_mapped = PJ_TRUE;
+
+		    /* Find other host candidates with the same (IPv6)
+		     * address, and replace it with the new (IPv4)
+		     * mapped address.
+		     */
+		    for (i = 0; i < comp->cand_cnt; ++i) {
+		        pj_sockaddr *a1, *a2;
+
+		        if (comp->cand_list[i].type != PJ_ICE_CAND_TYPE_HOST)
+		            continue;
+		        
+		        a1 = &comp->cand_list[i].addr;
+		        a2 = &cand->base_addr;
+		        if (pj_memcmp(pj_sockaddr_get_addr(a1),
+		       		      pj_sockaddr_get_addr(a2),
+		       		      pj_sockaddr_get_addr_len(a1)) == 0)
+		       	{
+		       	    pj_uint16_t port = pj_sockaddr_get_port(a1);
+		       	    pj_sockaddr_cp(a1, &info.mapped_addr);
+		       	    if (port != pj_sockaddr_get_port(a2))
+		       	        pj_sockaddr_set_port(a1, port);
+		       	    pj_sockaddr_cp(&comp->cand_list[i].base_addr, a1);
+		       	}
+		    }
+		    pj_sockaddr_cp(&cand->base_addr, &info.mapped_addr);
+		    pj_sockaddr_cp(&cand->rel_addr, &info.mapped_addr);
+		}
+		
 		/* Eliminate the srflx candidate if the address is
 		 * equal to other (host) candidates.
 		 */
@@ -1946,7 +2091,7 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	pj_turn_session_info rel_info;
 	char ipaddr[PJ_INET6_ADDRSTRLEN+8];
 	pj_ice_sess_cand *cand = NULL;
-	unsigned i;
+	unsigned i, cand_idx = 0xFF;
 
 	comp->turn[tp_idx].err_cnt = 0;
 
@@ -1962,6 +2107,7 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 		comp->cand_list[i].transport_id == data->transport_id)
 	    {
 		cand = &comp->cand_list[i];
+		cand_idx = i;
 		break;
 	    }
 	}
@@ -1979,16 +2125,85 @@ static void turn_on_state(pj_turn_sock *turn_sock, pj_turn_state_t old_state,
 	cand->status = PJ_SUCCESS;
 
 	/* Set default candidate to relay */
-	comp->default_cand = (unsigned)(cand - comp->cand_list);
+	if (comp->cand_list[comp->default_cand].type!=PJ_ICE_CAND_TYPE_RELAYED
+	    || (comp->ice_st->cfg.af != pj_AF_UNSPEC() &&
+	        comp->cand_list[comp->default_cand].addr.addr.sa_family
+	        != comp->ice_st->cfg.af))
+	{
+	    comp->default_cand = (unsigned)(cand - comp->cand_list);
+	}
+
+	/* Prefer IPv4 relay as default candidate for better connectivity
+	 * with IPv4 endpoints.
+	 */
+	/*
+	if (cand->addr.addr.sa_family != pj_AF_INET()) {
+	    for (i=0; i<comp->cand_cnt; ++i) {
+		if (comp->cand_list[i].type == PJ_ICE_CAND_TYPE_RELAYED &&
+		    comp->cand_list[i].addr.addr.sa_family == pj_AF_INET() &&
+		    comp->cand_list[i].status == PJ_SUCCESS)
+		{
+		    comp->default_cand = i;
+		    break;
+		}
+	    }
+	}
+	*/
 
 	PJ_LOG(4,(comp->ice_st->obj_name,
-		  "Comp %d: TURN allocation complete, relay address is %s",
-		  comp->comp_id,
+		  "Comp %d/%d: TURN allocation (tpid=%d) complete, "
+		  "relay address is %s",
+		  comp->comp_id, cand_idx, cand->transport_id,
 		  pj_sockaddr_print(&rel_info.relay_addr, ipaddr,
 				     sizeof(ipaddr), 3)));
 
 	sess_init_update(comp->ice_st);
 
+    } else if ((old_state == PJ_TURN_STATE_RESOLVING ||
+                old_state == PJ_TURN_STATE_RESOLVED) &&
+	       new_state == PJ_TURN_STATE_DESTROYING)
+    {
+	pj_ice_sess_cand *cand = NULL;
+	unsigned i, cand_idx = 0xFF;
+
+	/* DNS resolution or TURN transport creation/allocation
+	 * has failed.
+	 */
+	++comp->turn[tp_idx].err_cnt;
+
+	/* Unregister ourself from the TURN relay */
+	pj_turn_sock_set_user_data(turn_sock, NULL);
+	comp->turn[tp_idx].sock = NULL;
+
+	/* Wait until initialization completes */
+	pj_grp_lock_acquire(comp->ice_st->grp_lock);
+
+	/* Find relayed candidate in the component */
+	for (i=0; i<comp->cand_cnt; ++i) {
+	    if (comp->cand_list[i].type == PJ_ICE_CAND_TYPE_RELAYED &&
+		comp->cand_list[i].transport_id == data->transport_id)
+	    {
+		cand = &comp->cand_list[i];
+		cand_idx = i;
+		break;
+	    }
+	}
+
+	pj_grp_lock_release(comp->ice_st->grp_lock);
+
+	/* If the error happens during pj_turn_sock_create() or
+	 * pj_turn_sock_alloc(), the candidate hasn't been added
+	 * to the list.
+	 */
+	if (cand) {
+	    cand->status = PJ_ERESOLVE;
+	    PJ_LOG(4,(comp->ice_st->obj_name,
+		      "Comp %d/%d: TURN resolving (tpid=%d) failed",
+		      comp->comp_id, cand_idx, cand->transport_id));
+	}
+
+	sess_init_update(comp->ice_st);
+
     } else if (new_state >= PJ_TURN_STATE_DEALLOCATING) {
 	pj_turn_session_info info;
 
