@@ -1,4 +1,4 @@
-/* $Id: pjsua_app_legacy.c 5265 2016-03-17 03:37:13Z ming $ */
+/* $Id: pjsua_app_legacy.c 5461 2016-10-14 04:53:07Z ming $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -1378,7 +1378,7 @@ static void ui_sleep(char menuin[])
 	    return;
 	}
 
-	delay = pj_strtoul(&tmp);
+	delay = (int)pj_strtoul(&tmp);
 	if (delay < 0) delay = 0;
 	pj_thread_sleep(delay);
     }
