@@ -1,4 +1,4 @@
-/* $Id: siprtp.c 5366 2016-06-29 05:53:10Z nanang $ */
+/* $Id: siprtp.c 5535 2017-01-19 10:31:38Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -1397,14 +1397,12 @@ static void call_on_media_update( pjsip_inv_session *inv,
 				  pj_status_t status)
 {
     struct call *call;
-    pj_pool_t *pool;
     struct media_stream *audio;
     const pjmedia_sdp_session *local_sdp, *remote_sdp;
     struct codec *codec_desc = NULL;
     unsigned i;
 
     call = inv->mod_data[mod_siprtp.id];
-    pool = inv->dlg->pool;
     audio = &call->media[0];
 
     /* If this is a mid-call media update, then destroy existing media */
