@@ -1,4 +1,4 @@
-/* $Id: account.hpp 4957 2014-11-04 08:00:15Z nanang $ */
+/* $Id: account.hpp 5649 2017-09-15 05:32:08Z riza $ */
 /*
  * Copyright (C) 2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -69,6 +69,17 @@ struct AccountRegConfig : public PersistentObject
      */
     SipHeaderVector	headers;
 
+    /**
+     * Additional parameters that will be appended in the Contact header
+     * of the registration requests. This will be appended after
+     * \a AccountSipConfig.contactParams;
+     *
+     * The parameters should be preceeded by semicolon, and all strings must
+     * be properly escaped. Example:
+     *	 ";my-param=X;another-param=Hi%20there"
+     */
+    string	    	contactParams;
+
     /**
      * Optional interval for registration, in seconds. If the value is zero,
      * default interval will be used (PJSUA_REG_INTERVAL, 300 seconds).
@@ -457,6 +468,13 @@ struct AccountNatConfig : public PersistentObject
      */
     pjsua_stun_use 	mediaStunUse;
 
+    /**
+     * Specify NAT64 options.
+     *
+     * Default: PJSUA_NAT64_DISABLED
+     */
+    pjsua_nat64_opt 	nat64Opt;
+
     /**
      * Enable ICE for the media transport.
      *
@@ -831,6 +849,21 @@ struct AccountVideoConfig : public PersistentObject
      */
     unsigned			rateControlBandwidth;
 
+    /**
+     * The number of keyframe to be sent after the stream is created.
+     *
+     * Default: PJMEDIA_VID_STREAM_START_KEYFRAME_CNT
+     */
+    unsigned			    startKeyframeCount;
+
+    /**
+     * The keyframe sending interval after the stream is created.
+     *
+     * Default: PJMEDIA_VID_STREAM_START_KEYFRAME_INTERVAL_MSEC
+     */
+    unsigned			    startKeyframeInterval;
+
+
 public:
     /**
      * Read this object from a container node.
@@ -847,6 +880,56 @@ public:
     virtual void writeObject(ContainerNode &node) const throw(Error);
 };
 
+/**
+ * Account config specific to IP address change.
+ */
+typedef struct AccountIpChangeConfig
+{    
+    /**
+     * Shutdown the transport used for account registration. If this is set to
+     * PJ_TRUE, the transport will be shutdown altough it's used by multiple
+     * account. Shutdown transport will be followed by re-Registration if
+     * AccountConfig.natConfig.contactRewriteUse is enabled.
+     *
+     * Default: true
+     */
+    bool    		shutdownTp;
+
+    /**
+     * Hangup active calls associated with the acount. If this is set to true, 
+     * then the calls will be hang up.
+     *
+     * Default: false
+     */
+    bool		hangupCalls;
+
+    /**
+     * Specify the call flags used in the re-INVITE when \a hangupCalls is set 
+     * to false. If this is set to 0, no re-INVITE will be sent. The 
+     * re-INVITE will be sent after re-Registration is finished.
+     *
+     * Default: PJSUA_CALL_REINIT_MEDIA | PJSUA_CALL_UPDATE_CONTACT |
+     *          PJSUA_CALL_UPDATE_VIA
+     */
+    unsigned		reinviteFlags;
+
+public:
+    /**
+     * Read this object from a container node.
+     *
+     * @param node		Container to read values from.
+     */
+    virtual void readObject(const ContainerNode &node) throw(Error);
+
+    /**
+     * Write this object to a container node.
+     *
+     * @param node		Container to write values to.
+     */
+    virtual void writeObject(ContainerNode &node) const throw(Error);
+    
+} AccountIpChangeConfig;
+
 /**
  * Account configuration.
  */
@@ -908,6 +991,11 @@ struct AccountConfig : public PersistentObject
      */
     AccountVideoConfig	videoConfig;
 
+    /**
+     * IP Change settings.
+     */
+    AccountIpChangeConfig ipChangeConfig;
+
 public:
     /**
      * Default constructor will initialize with default values.
