@@ -1,4 +1,4 @@
-/* $Id: evsub.c 5351 2016-06-21 08:56:42Z nanang $ */
+/* $Id: evsub.c 5558 2017-02-20 01:29:21Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -412,7 +412,9 @@ PJ_DEF(pj_status_t) pjsip_evsub_register_pkg( pjsip_module *pkg_mod,
     unsigned i;
 
     PJ_ASSERT_RETURN(pkg_mod && event_name, PJ_EINVAL);
-    PJ_ASSERT_RETURN(accept_cnt < PJ_ARRAY_SIZE(pkg->pkg_accept->values), 
+    
+    /* Make sure accept_cnt < PJ_ARRAY_SIZE(pkg->pkg_accept->values) */
+    PJ_ASSERT_RETURN(accept_cnt <= PJSIP_GENERIC_ARRAY_MAX_COUNT, 
 		     PJ_ETOOMANY);
 
     /* Make sure evsub module has been initialized */
@@ -507,10 +509,9 @@ static void set_timer( pjsip_evsub *sub, int timer_id,
 	sub->timer.id = TIMER_TYPE_NONE;
     }
 
-    if (timer_id != TIMER_TYPE_NONE) {
+    if (timer_id != TIMER_TYPE_NONE && seconds > 0) {
 	pj_time_val timeout;
 
-	PJ_ASSERT_ON_FAIL(seconds > 0, return);
 	PJ_ASSERT_ON_FAIL(timer_id>TIMER_TYPE_NONE && timer_id<TIMER_TYPE_MAX,
 			  return);
 
@@ -527,6 +528,15 @@ static void set_timer( pjsip_evsub *sub, int timer_id,
 }
 
 
+/*
+ * Set event subscription UAS timout.
+ */
+PJ_DEF(void) pjsip_evsub_uas_set_timeout(pjsip_evsub *sub, pj_uint32_t seconds)
+{
+    set_timer(sub, TIMER_TYPE_UAS_TIMEOUT, (pj_int32_t)seconds);
+}
+
+
 /*
  * Destructor.
  */
