@@ -1,4 +1,4 @@
-/* $Id: sip_dialog.c 5373 2016-06-30 08:23:08Z ming $ */
+/* $Id: sip_dialog.c 5608 2017-06-20 04:12:09Z riza $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -92,6 +92,12 @@ static pj_status_t create_dialog( pjsip_user_agent *ua,
     pj_list_init(&dlg->inv_hdr);
     pj_list_init(&dlg->rem_cap_hdr);
 
+    /* Init client authentication session. */
+    status = pjsip_auth_clt_init(&dlg->auth_sess, dlg->endpt,
+				 dlg->pool, 0);
+    if (status != PJ_SUCCESS)
+	goto on_error;
+
     status = pj_mutex_create_recursive(pool, dlg->obj_name, &dlg->mutex_);
     if (status != PJ_SUCCESS)
 	goto on_error;
@@ -163,23 +169,25 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uac( pjsip_user_agent *ua,
 
 	param = uri->header_param.next;
 	while (param != &uri->header_param) {
-	    pjsip_hdr *hdr;
-	    int c;
+	    if (param->value.ptr) {
+		pjsip_hdr *hdr;
+		int c;
 
-	    c = param->value.ptr[param->value.slen];
-	    param->value.ptr[param->value.slen] = '\0';
+		c = param->value.ptr[param->value.slen];
+		param->value.ptr[param->value.slen] = '\0';
 
-	    hdr = (pjsip_hdr*)
-	    	  pjsip_parse_hdr(dlg->pool, &param->name, param->value.ptr,
-				  param->value.slen, NULL);
+		hdr = (pjsip_hdr*)
+		    pjsip_parse_hdr(dlg->pool, &param->name, param->value.ptr,
+				    param->value.slen, NULL);
 
-	    param->value.ptr[param->value.slen] = (char)c;
+		param->value.ptr[param->value.slen] = (char)c;
 
-	    if (hdr == NULL) {
-		status = PJSIP_EINVALIDURI;
-		goto on_error;
+		if (hdr == NULL) {
+		    status = PJSIP_EINVALIDURI;
+		    goto on_error;
+		}
+		pj_list_push_back(&dlg->inv_hdr, hdr);
 	    }
-	    pj_list_push_back(&dlg->inv_hdr, hdr);
 
 	    param = param->next;
 	}
@@ -283,12 +291,6 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_uac( pjsip_user_agent *ua,
     /* Initial route set is empty. */
     pj_list_init(&dlg->route_set);
 
-    /* Init client authentication session. */
-    status = pjsip_auth_clt_init(&dlg->auth_sess, dlg->endpt,
-				 dlg->pool, 0);
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
     /* Register this dialog to user agent. */
     status = pjsip_ua_register_dlg( ua, dlg );
     if (status != PJ_SUCCESS)
@@ -324,7 +326,7 @@ pj_status_t create_uas_dialog( pjsip_user_agent *ua,
     pjsip_rr_hdr *rr;
     pjsip_transaction *tsx = NULL;
     pj_str_t tmp;
-    enum { TMP_LEN=128};
+    enum { TMP_LEN=PJSIP_MAX_URL_SIZE };
     pj_ssize_t len;
     pjsip_dialog *dlg;
 
@@ -506,12 +508,6 @@ pj_status_t create_uas_dialog( pjsip_user_agent *ua,
     }
     dlg->route_set_frozen = PJ_TRUE;
 
-    /* Init client authentication session. */
-    status = pjsip_auth_clt_init(&dlg->auth_sess, dlg->endpt,
-				 dlg->pool, 0);
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
     /* Increment the dialog's lock since tsx may cause the dialog to be
      * destroyed prematurely (such as in case of transport error).
      */
@@ -1130,6 +1126,9 @@ static pj_status_t dlg_create_request_throw( pjsip_dialog *dlg,
     if (status != PJ_SUCCESS)
 	return status;
 
+    /* Put this dialog in tdata's mod_data */
+    tdata->mod_data[dlg->ua->id] = dlg;
+
     /* Just copy dialog route-set to Route header.
      * The transaction will do the processing as specified in Section 12.2.1
      * of RFC 3261 in function tsx_process_route() in sip_transaction.c.
@@ -1230,6 +1229,9 @@ PJ_DEF(pj_status_t) pjsip_dlg_send_request( pjsip_dialog *dlg,
     /* Lock and increment session */
     pjsip_dlg_inc_lock(dlg);
 
+    /* Put this dialog in tdata's mod_data */
+    tdata->mod_data[dlg->ua->id] = dlg;
+
     /* If via_addr is set, use this address for the Via header. */
     if (dlg->via_addr.host.slen > 0) {
         tdata->via_addr = dlg->via_addr;
@@ -1412,6 +1414,9 @@ PJ_DEF(pj_status_t) pjsip_dlg_create_response(	pjsip_dialog *dlg,
     /* Lock the dialog. */
     pjsip_dlg_inc_lock(dlg);
 
+    /* Put this dialog in tdata's mod_data */
+    tdata->mod_data[dlg->ua->id] = dlg;
+
     dlg_beautify_response(dlg, PJ_FALSE, st_code, tdata);
 
     /* Unlock the dialog. */
