@@ -1,4 +1,4 @@
-/* $Id: sip_multipart.c 5170 2015-08-25 08:45:46Z nanang $ */
+/* $Id: sip_multipart.c 5594 2017-05-22 03:53:35Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -45,6 +45,7 @@ struct multipart_data
 {
     pj_str_t	    	  boundary;
     pjsip_multipart_part  part_head;
+    pj_str_t		  raw_data;
 };
 
 
@@ -67,6 +68,7 @@ static int multipart_print_body(struct pjsip_msg_body *msg_body,
 	enum { CLEN_SPACE = 5 };
 	char *clen_pos;
 	const pjsip_hdr *hdr;
+	pj_bool_t ctype_printed = PJ_FALSE;
 
 	clen_pos = NULL;
 
@@ -82,19 +84,24 @@ static int multipart_print_body(struct pjsip_msg_body *msg_body,
 	hdr = part->hdr.next;
 	while (hdr != &part->hdr) {
 	    int printed = pjsip_hdr_print_on((pjsip_hdr*)hdr, p,
-	                                     SIZE_LEFT()-2);
+					     SIZE_LEFT()-2);
 	    if (printed < 0)
 		return -1;
 	    p += printed;
 	    *p++ = '\r';
 	    *p++ = '\n';
+
+	    if (!ctype_printed && hdr->type == PJSIP_H_CONTENT_TYPE)
+		ctype_printed = PJ_TRUE;	    
+
 	    hdr = hdr->next;
 	}
 
 	/* Automaticly adds Content-Type and Content-Length headers, only
-	 * if content_type is set in the message body.
+	 * if content_type is set in the message body and haven't been printed.
 	 */
-	if (part->body && part->body->content_type.type.slen) {
+	if (part->body && part->body->content_type.type.slen && !ctype_printed) 
+	{
 	    pj_str_t ctype_hdr = { "Content-Type: ", 14};
 	    const pjsip_media_type *media = &part->body->content_type;
 
@@ -602,6 +609,12 @@ PJ_DEF(pjsip_msg_body*) pjsip_multipart_parse(pj_pool_t *pool,
 
     body = pjsip_multipart_create(pool, ctype, &boundary);
 
+    /* Save full raw body */
+    {
+	struct multipart_data *mp = (struct multipart_data*)body->data;
+	pj_strset(&mp->raw_data, buf, len);
+    }
+
     for (;;) {
 	char *start_body, *end_body;
 	pjsip_multipart_part *part;
@@ -640,13 +653,18 @@ PJ_DEF(pjsip_msg_body*) pjsip_multipart_parse(pj_pool_t *pool,
 
 	end_body = curptr;
 
-	/* The newline preceeding the delimiter is conceptually part of
-	 * the delimiter, so trim it from the body.
+	/* Note that when body is empty, end_body will be equal
+	 * to start_body.
 	 */
-	if (*(end_body-1) == '\n')
-	    --end_body;
-	if (*(end_body-1) == '\r')
-	    --end_body;
+        if (end_body > start_body) {
+	    /* The newline preceeding the delimiter is conceptually part of
+	     * the delimiter, so trim it from the body.
+	     */
+	    if (*(end_body-1) == '\n')
+	        --end_body;
+	    if (*(end_body-1) == '\r')
+	        --end_body;
+        }
 
 	/* Now that we have determined the part's boundary, parse it
 	 * to get the header and body part of the part.
@@ -661,3 +679,26 @@ PJ_DEF(pjsip_msg_body*) pjsip_multipart_parse(pj_pool_t *pool,
     return body;
 }
 
+
+PJ_DEF(pj_status_t) pjsip_multipart_get_raw( pjsip_msg_body *mp,
+					     pj_str_t *boundary,
+					     pj_str_t *raw_data)
+{
+    struct multipart_data *m_data;
+
+    /* Must specify mandatory params */
+    PJ_ASSERT_RETURN(mp, PJ_EINVAL);
+
+    /* mp must really point to an actual multipart msg body */
+    PJ_ASSERT_RETURN(mp->print_body==&multipart_print_body, PJ_EINVAL);
+
+    m_data = (struct multipart_data*)mp->data;
+
+    if (boundary)
+	*boundary = m_data->boundary;
+
+    if (raw_data)
+	*raw_data = m_data->raw_data;
+
+    return PJ_SUCCESS;
+}
