@@ -1,4 +1,4 @@
-/* $Id: sip_transport_udp.c 5311 2016-05-20 04:17:00Z ming $ */
+/* $Id: sip_transport_udp.c 5649 2017-09-15 05:32:08Z riza $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -76,6 +76,7 @@ struct udp_transport
     pjsip_rx_data     **rdata;
     int			is_closing;
     pj_bool_t		is_paused;
+    int			read_loop_spin;
 
     /* Group lock to be used by UDP transport and ioqueue key */
     pj_grp_lock_t      *grp_lock;
@@ -129,15 +130,17 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
     int i;
     pj_status_t status;
 
+    ++tp->read_loop_spin;
+
     /* Don't do anything if transport is closing. */
     if (tp->is_closing) {
 	tp->is_closing++;
-	return;
+	goto on_return;
     }
 
     /* Don't do anything if transport is being paused. */
     if (tp->is_paused)
-	return;
+	goto on_return;
 
     /*
      * The idea of the loop is to process immediate data received by
@@ -227,7 +230,7 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	 * is still processing a SIP message.
 	 */
 	if (tp->is_paused)
-	    return;
+	    break;
 
 	/* Read next packet. */
 	bytes_read = sizeof(rdata->pkt_info.packet);
@@ -245,6 +248,10 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	} else if (status == PJ_EPENDING) {
 	    break;
 
+	} else if (status == PJ_ECANCELLED) {
+	    /* Socket is closing, quit loop */
+	    break;
+
 	} else {
 
 	    if (i < MAX_IMMEDIATE_PACKET) {
@@ -276,6 +283,9 @@ static void udp_on_read_complete( pj_ioqueue_key_t *key,
 	    }
 	}
     }
+
+on_return:
+    --tp->read_loop_spin;
 }
 
 /*
@@ -623,13 +633,16 @@ static pj_status_t register_to_ioqueue(struct udp_transport *tp)
     if (tp->key != NULL)
     	return PJ_SUCCESS;
 
-    /* Create group lock */
-    status = pj_grp_lock_create(tp->base.pool, NULL, &tp->grp_lock);
-    if (status != PJ_SUCCESS)
-	return status;
+    /* Create group lock if not yet (don't need to do so on UDP restart) */
+    if (!tp->grp_lock) {
+	status = pj_grp_lock_create(tp->base.pool, NULL, &tp->grp_lock);
+	if (status != PJ_SUCCESS)
+	    return status;
 
-    pj_grp_lock_add_ref(tp->grp_lock);
-    pj_grp_lock_add_handler(tp->grp_lock, tp->base.pool, tp, &udp_on_destroy);
+	pj_grp_lock_add_ref(tp->grp_lock);
+	pj_grp_lock_add_handler(tp->grp_lock, tp->base.pool, tp,
+				&udp_on_destroy);
+    }
     
     /* Register to ioqueue. */
     ioqueue = pjsip_endpt_get_ioqueue(tp->base.endpt);
@@ -687,7 +700,7 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
 {
     pj_pool_t *pool;
     struct udp_transport *tp;
-    const char *format, *ipv6_quoteb, *ipv6_quotee;
+    const char *format, *ipv6_quoteb = "", *ipv6_quotee = "";
     unsigned i;
     pj_status_t status;
 
@@ -696,12 +709,18 @@ static pj_status_t transport_attach( pjsip_endpoint *endpt,
 
     /* Object name. */
     if (type & PJSIP_TRANSPORT_IPV6) {
+        pj_in6_addr dummy6;
+
 	format = "udpv6%p";
-	ipv6_quoteb = "[";
-	ipv6_quotee = "]";
+	/* We don't need to add quote if the transport type is IPv6, but
+	 * actually translated to IPv4.
+	 */
+        if (pj_inet_pton(pj_AF_INET6(), &a_name->host, &dummy6)==PJ_SUCCESS) {
+	    ipv6_quoteb = "[";
+	    ipv6_quotee = "]";
+	}
     } else {
 	format = "udp%p";
-	ipv6_quoteb = ipv6_quotee = "";
     }
 
     /* Create pool. */
@@ -1061,10 +1080,21 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_pause(pjsip_transport *transport,
  *  - if socket is not specified, create and replace.
  */
 PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
-					        unsigned option,
+						unsigned option,
 						pj_sock_t sock,
 						const pj_sockaddr_in *local,
 						const pjsip_host_port *a_name)
+{
+    return pjsip_udp_transport_restart2(transport, option, sock, 
+					(pj_sockaddr*)local, a_name);
+}
+
+
+PJ_DEF(pj_status_t) pjsip_udp_transport_restart2(pjsip_transport *transport,
+					         unsigned option,
+					         pj_sock_t sock,
+					         const pj_sockaddr *local,
+					         const pjsip_host_port *a_name)
 {
     struct udp_transport *tp;
     pj_status_t status;
@@ -1075,6 +1105,11 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 
     tp = (struct udp_transport*) transport;
 
+    /* Pause the transport first, so that any active read loop spin will
+     * quit as soon as possible.
+     */
+    tp->is_paused = PJ_TRUE;
+    
     if (option & PJSIP_UDP_TRANSPORT_DESTROY_SOCKET) {
 	char addr_buf[PJ_INET6_ADDRSTRLEN];
 	pjsip_host_port bound_name;
@@ -1097,8 +1132,8 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 
 	/* Create the socket if it's not specified */
 	if (sock == PJ_INVALID_SOCKET) {
-	    status = create_socket(pj_AF_INET(), local, 
-				   sizeof(pj_sockaddr_in), &sock);
+	    status = create_socket(local->addr.sa_family, local, 
+				   pj_sockaddr_get_len(local), &sock);
 	    if (status != PJ_SUCCESS)
 		return status;
 	}
@@ -1138,6 +1173,11 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 	    udp_set_pub_name(tp, a_name);
     }
 
+    /* Make sure all udp_on_read_complete() loop spin are stopped */
+    do {
+	pj_thread_sleep(1);
+    } while (tp->read_loop_spin);
+
     /* Re-register new or existing socket to ioqueue. */
     status = register_to_ioqueue(tp);
     if (status != PJ_SUCCESS) {
@@ -1160,4 +1200,3 @@ PJ_DEF(pj_status_t) pjsip_udp_transport_restart(pjsip_transport *transport,
 
     return PJ_SUCCESS;
 }
-
